/**
 * @license
 * Copyright Tyler Technologies, Inc. 
 * License: Apache-2.0
 */
import { BaseComponent } from '../core/base/base-component';
import { IWithDefaultAria } from '../core/mixins/internals/with-default-aria';
import { IWithElementInternals } from '../core/mixins/internals/with-element-internals';
import { IDismissible, IDismissibleStackState, tryDismiss } from '../core/utils/dismissible-stack';
import { DialogAnimationType, DialogMode, DialogPlacement, DialogPositionStrategy, DialogPreset, DialogSizeStrategy, DialogType, IDialogBeforeCloseEventData, IDialogMoveEventData, IDialogMoveStartEventData, dialogStack, hideBackdrop, showBackdrop } from './dialog-constants';
export interface IDialogProperties {
    open: boolean;
    mode: DialogMode;
    type: DialogType;
    animationType: DialogAnimationType;
    preset: DialogPreset;
    persistent: boolean;
    fullscreen: boolean;
    fullscreenThreshold: number;
    trigger: string;
    triggerElement: HTMLElement | null;
    positionStrategy: DialogPositionStrategy;
    sizeStrategy: DialogSizeStrategy;
    placement: DialogPlacement;
    moveable: boolean;
    label: string;
    description: string;
    [showBackdrop](): void;
    [hideBackdrop](): void;
}
export interface IDialogComponent extends IDialogProperties, IWithDefaultAria, IWithElementInternals, IDismissible {
    show(): void;
    hide(): void;
}
declare global {
    interface HTMLElementTagNameMap {
        'forge-dialog': IDialogComponent;
    }
    interface HTMLElementEventMap {
        'forge-dialog-open': CustomEvent<void>;
        'forge-dialog-close': CustomEvent<void>;
        'forge-dialog-before-close': CustomEvent<IDialogBeforeCloseEventData>;
        'forge-dialog-move-start': CustomEvent<IDialogMoveStartEventData>;
        'forge-dialog-move': CustomEvent<IDialogMoveEventData>;
        'forge-dialog-move-end': CustomEvent<void>;
        'forge-dialog-fullscreen-change': CustomEvent<boolean>;
    }
}
declare const DialogComponent_base: import("..").AbstractConstructor<import("../core/mixins/internals/with-default-aria").WithDefaultAriaContract> & import("..").AbstractConstructor<import("../core/mixins/internals/with-element-internals").WithElementInternalsContract> & typeof BaseComponent;
/**
 * @tag forge-dialog
 *
 * @summary Dialogs are temporary UI elements that are used to display information, ask for input, or confirm actions.
 *
 * @dependency forge-backdrop
 *
 * @globalconfig mode
 * @globalconfig animationType
 * @globalconfig positionStrategy
 * @globalconfig sizeStrategy
 * @globalconfig persistent
 * @globalconfig moveable
 * @globalconfig fullscreenThreshold
 *
 * @event {CustomEvent<void>} forge-dialog-open - Dispatched when the dialog is opened.
 * @event {CustomEvent<void>} forge-dialog-close - Dispatched when the dialog is closed.
 * @event {CustomEvent<IDialogBeforeCloseEventData>} forge-dialog-before-close - Dispatched before the dialog is closed. This event is cancelable.
 * @event {CustomEvent<IDialogMoveStartEventData>} forge-dialog-move-start - Dispatched when the dialog is first moved.
 * @event {CustomEvent<IDialogMoveEventData>} forge-dialog-move - Dispatched when the dialog is being moved.
 * @event {CustomEvent<void>} forge-dialog-move-end - Dispatched when the dialog is done being moved.
 * @event {CustomEvent<boolean>} forge-dialog-fullscreen-change - Dispatched when the dialog's fullscreen state changes.
 *
 * @cssproperty --forge-dialog-background - The background color of the dialog.
 * @cssproperty --forge-dialog-shape - The shape of the dialog.
 * @cssproperty --forge-dialog-elevation - The elevation of the dialog.
 * @cssproperty --forge-dialog-spacing - The spacing between elements inside the dialog.
 * @cssproperty --forge-dialog-block-start-spacing - The spacing at the start of the dialog block.
 * @cssproperty --forge-dialog-block-end-spacing - The spacing at the end of the dialog block.
 * @cssproperty --forge-dialog-inline-start-spacing - The spacing at the start of the dialog inline.
 * @cssproperty --forge-dialog-inline-end-spacing - The spacing at the end of the dialog inline.
 * @cssproperty --forge-dialog-padding - The padding of the dialog.
 * @cssproperty --forge-dialog-width - The width of the dialog.
 * @cssproperty --forge-dialog-height - The height of the dialog.
 * @cssproperty --forge-dialog-min-width - The minimum width of the dialog.
 * @cssproperty --forge-dialog-max-width - The maximum width of the dialog.
 * @cssproperty --forge-dialog-min-height - The minimum height of the dialog.
 * @cssproperty --forge-dialog-max-height - The maximum height of the dialog.
 * @cssproperty --forge-dialog-z-index - The z-index of the dialog.
 * @cssproperty --forge-dialog-move-handle-color - The color of the move handle.
 * @cssproperty --forge-dialog-move-handle-size - The size of the move handle.
 * @cssproperty --forge-dialog-move-handle-hover-cursor - The cursor style when hovering over the move handle.
 * @cssproperty --forge-dialog-move-handle-active-cursor - The cursor style when the move handle is active.
 * @cssproperty --forge-dialog-move-handle-spacing - The spacing around the move handle.
 * @cssproperty --forge-dialog-move-transition-duration - The duration of the move transition.
 * @cssproperty --forge-dialog-move-transition-easing - The easing function of the move transition.
 * @cssproperty --forge-dialog-moving-opacity - The opacity of the dialog when it is being moved.
 * @cssproperty --forge-dialog-enter-animation-duration - The duration of the enter animation.
 * @cssproperty --forge-dialog-enter-animation-easing - The easing function of the enter animation.
 * @cssproperty --forge-dialog-exit-animation-duration - The duration of the exit animation.
 * @cssproperty --forge-dialog-exit-animation-easing - The easing function of the exit animation.
 * @cssproperty --forge-dialog-zoom-opacity - The opacity of the dialog during zoom animation.
 * @cssproperty --forge-dialog-zoom-scale - The scale of the dialog during zoom animation.
 * @cssproperty --forge-dialog-fade-opacity - The opacity of the dialog during fade animation.
 * @cssproperty --forge-dialog-slide-opacity - The opacity of the dialog during slide animation.
 * @cssproperty --forge-dialog-slide-translate - The translation distance of the dialog during slide animation.
 * @cssproperty --forge-dialog-backdrop-opacity - The opacity of the dialog backdrop.
 * @cssproperty --forge-dialog-nonmodal-elevation - The elevation of non-modal dialogs.
 * @cssproperty --forge-dialog-fullscreen-enter-animation-duration - The duration of the enter animation for fullscreen dialogs.
 * @cssproperty --forge-dialog-fullscreen-exit-animation-duration - The duration of the exit animation for fullscreen dialogs.
 * @cssproperty --forge-dialog-position-x - The x-axis position of the dialog.
 * @cssproperty --forge-dialog-position-y - The y-axis position of the dialog.
 * @cssproperty --forge-dialog-preset-sheet-enter-animation-duration - The duration of the enter animation for preset sheet dialogs.
 * @cssproperty --forge-dialog-preset-sheet-exit-animation-duration - The duration of the exit animation for preset sheet dialogs.
 *
 * @csspart root - The dialog container element.
 * @csspart backdrop - The backdrop element.
 * @csspart surface - The dialog surface element.
 * @csspart move-handle-container - The alignment container for the move handle.
 * @csspart move-handle - The move handle element.
 * @csspart move-handle-icon - The move handle icon element.
 *
 * @slot - The content of the dialog.
 * @slot move-handle - The move handle content.
 *
 * @cssclass forge-dialog - Apply to the root `<dialog>` element _(required)_.
 * @cssclass forge-dialog--fullscreen - Renders the dialog in fullscreen mode.
 * @cssclass forge-dialog--top - Places the dialog at the top of the screen.
 * @cssclass forge-dialog--top-right - Places the dialog at the top right of the screen.
 * @cssclass forge-dialog--top-left - Places the dialog at the top left of the screen.
 * @cssclass forge-dialog--right - Places the dialog at the right of the screen.
 * @cssclass forge-dialog--left - Places the dialog at the left of the screen.
 * @cssclass forge-dialog--bottom - Places the dialog at the bottom of the screen.
 * @cssclass forge-dialog--bottom-right - Places the dialog at the bottom right of the screen.
 * @cssclass forge-dialog--bottom-left - Places the dialog at the bottom left of the screen.
 * @cssclass forge-dialog--custom - Allows for custom placement of the dialog.
 * @cssclass forge-dialog--bottom-sheet - Renders the dialog as a bottom sheet.
 * @cssclass forge-dialog--top-sheet - Renders the dialog as a top sheet.
 * @cssclass forge-dialog--left-sheet - Renders the dialog as a left sheet.
 * @cssclass forge-dialog--right-sheet - Renders the dialog as a right sheet.
 * @cssclass forge-dialog--animation-fade - Sets to dialog to open and close with a fade animation.
 * @cssclass forge-dialog--animation-slide - Sets to dialog to open and close with a slide animation.
 * @cssclass forge-dialog--animation-none - Sets to dialog to open and close without any animation.
 * @cssclass forge-dialog--moveable - Applies moveable dialog styles.
 * @cssclass forge-dialog--moving - Apply to the dialog when it is being moved.
 * @cssclass forge-dialog--moved - Apply to the dialog after it has been moved.
 * @cssclass forge-dialog__move-handle - Apply to the move handle element.
 * @cssclass forge-dialog__move-handle-container - Apply to the parent of the move handle element.
 */
export declare class DialogComponent extends DialogComponent_base implements IDialogComponent {
    static get observedAttributes(): string[];
    /**
     * @internal
     * Contains all the dialogs that are currently open.
     */
    static readonly [dialogStack]: Set<IDialogComponent>;
    private _core;
    /** @internal */
    [hideBackdrop](): void;
    /** @internal */
    [showBackdrop](): void;
    [tryDismiss](_state?: IDismissibleStackState<string> | undefined): boolean;
    constructor();
    connectedCallback(): void;
    disconnectedCallback(): void;
    attributeChangedCallback(name: string, oldValue: string, newValue: string): void;
    /**
     * Indicates whether the dialog is open.
     * @default false
     * @attribute
     */
    open: boolean;
    /**
     * The mode of the dialog.
     * @default 'modal'
     * @attribute
     */
    mode: DialogMode;
    /**
     * The type of the dialog.
     * @default 'dialog'
     * @attribute
     */
    type: DialogType;
    /**
     * The animation type of the dialog.
     * @default 'zoom'
     * @attribute animation-type
     */
    animationType: DialogAnimationType;
    /**
     * The preset design that the dialog will apply.
     * @default 'dialog'
     * @attribute
     */
    preset: DialogPreset;
    /**
     * Indicates whether the dialog is dismissible via escape and backdrop click or not.
     * @default false
     * @attribute
     */
    persistent: boolean;
    /**
     * Indicates whether the dialog is fullscreen or not.
     * @default false
     * @attribute
     */
    fullscreen: boolean;
    /**
     * The screen width at which the dialog will switch to fullscreen.
     * @default 599
     * @attribute fullscreen-threshold
     */
    fullscreenThreshold: number;
    /**
     * The selector of the element that triggers the dialog.
     * @default ''
     * @attribute
     */
    trigger: string;
    /**
     * The element that triggers the dialog.
     * @default null
     */
    triggerElement: HTMLElement | null;
    /**
     * Indicates whether the dialog is moveable or not.
     * @default false
     * @attribute
     */
    moveable: boolean;
    /**
     * Controls whether the dialog is rendered relative to the viewport or its nearest containing block.
     * @default 'viewport'
     * @attribute position-strategy
     */
    positionStrategy: DialogPositionStrategy;
    /**
     * Controls the block and/or inline size of the dialog. Defaults to the size of the content it contains.
     * @default 'content'
     * @attribute size-strategy
     */
    sizeStrategy: DialogSizeStrategy;
    /**
     * The placement of the dialog.
     * @default 'center'
     * @attribute
     */
    placement: DialogPlacement;
    /**
     * The accessible label of the dialog.
     * @default ''
     * @attribute
     */
    label: string;
    /**
     * The accessible description of the dialog.
     * @default ''
     * @attribute
     */
    description: string;
    /** Shows the dialog. */
    show(): void;
    /** Hides the dialog. */
    hide(): void;
}
export {};
