/**
 * @license
 * Copyright Tyler Technologies, Inc. 
 * License: Apache-2.0
 */
import { CUSTOM_ELEMENT_NAME_PROPERTY } from '@tylertech/forge-core';
import { PropertyValues, TemplateResult } from 'lit';
import { MeterDensity, MeterDirection, MeterInnerShape, MeterShape } from '../meter/meter';
import { BaseLitElement } from '../../core/base/base-lit-element';
export declare const METER_GROUP_TAG_NAME: keyof HTMLElementTagNameMap;
/**
 * @tag forge-meter-group
 *
 * @summary Meter groups display several meters together on one track.
 *
 * @state vertical - Applied when the meter group is oriented vertically.
 *
 * @cssproperty --forge-meter-group-background - The background color of the meter group.
 * @cssproperty --forge-meter-group-height - The block size of the meter group.
 * @cssproperty --forge-meter-group-shape - The border radius of the meter group.
 * @cssproperty --forge-meter-group-tickmarks - The number of tickmarks to display.
 * @cssproperty --forge-meter-group-tickmark-color - The color of the tickmarks.
 * @cssproperty --forge-meter-group-tickmark-opacity - The opacity of the tickmarks.
 *
 * @csspart root - The root container element.
 * @csspart track - The element comprising the meter group's background.
 *
 * @slot - The default slot for grouped `<forge-meter>` elements.
 * @slot label - Positions a label above the meter group.
 * @slot value - A textual representation of the meter's value.
 */
export declare class MeterGroupComponent extends BaseLitElement {
    static styles: import("lit").CSSResult;
    static formAssociated: boolean;
    /** @deprecated Used for compatibility with legacy Forge @customElement decorator. */
    static [CUSTOM_ELEMENT_NAME_PROPERTY]: keyof HTMLElementTagNameMap;
    /**
     * The minimum value of each meter in the group.
     * @default 0
     * @attribute
     */
    min: number;
    /**
     * The maximum value of each meter in the group.
     * @default 1
     * @attribute
     */
    max: number;
    /**
     * Whether to display tickmarks.
     * @default false
     * @attribute
     */
    tickmarks: boolean;
    /**
     * Whether the meter is oriented horizontally or vertically.
     * @default 'horizontal'
     * @attribute
     */
    direction: MeterDirection;
    /**
     * The density of the meter group.
     * @default 'default'
     * @attribute
     */
    density: MeterDensity;
    /**
     * The shape of the meter group.
     * @default 'default'
     * @attribute
     */
    shape: MeterShape;
    /**
     * The shape of each meter in the group.
     * @default 'default'
     * @attribute inner-shape
     */
    innerShape: MeterInnerShape;
    get labels(): NodeList;
    get form(): HTMLFormElement | null;
    private _hasSlottedHeadingContent;
    private _labelNodes;
    private _valueNodes;
    private _meters;
    private _internals;
    constructor();
    connectedCallback(): void;
    willUpdate(changedProperties: PropertyValues<this>): void;
    render(): TemplateResult;
    /**
     * Applies the min and max values to each meter in the group. These should always be set on the
     * meter group component to ensure that all meters are in sync.
     */
    private _syncMeters;
    /**
     * Debounce the handler to reduce the number of times it's called during rapid changes,
     * especially when the component is first connected.
     */
    private _debounceMeterChange;
    /**
     * Handles changes to the meters in the group, updating their arrangement relative to each other.
     */
    private _handleMeterChange;
    /**
     * Checks whether the meter group has any slotted heading content.
     */
    private _handleHeadingSlotChange;
}
declare global {
    interface HTMLElementTagNameMap {
        'forge-meter-group': MeterGroupComponent;
    }
}
